#include <algorithm>
#include <bitset>
#include <cstdio>
#include <cstdlib>
#include <cstdarg>

using namespace std;

const int EXIT_CODE_VALID_TEST_OUTPUT = 0;
const int EXIT_CODE_CHECKER_ERROR = 1;
const int EXIT_CODE_INVALID_TEST_OUTPUT = 0;

const int MAXN = 10000;

int n, m;
bitset<MAXN> neighbors[MAXN];

void input_fail(const char *format, ...) {
  va_list args;
  va_start(args, format);
  printf("0.0\n");
  fprintf(stderr, "Bug: Error in input: ");
  vfprintf(stderr, format, args);
  fprintf(stderr, "\n");
  va_end(args);
  exit(EXIT_CODE_CHECKER_ERROR);
}

void output_fail(bool is_test, const char *format, ...) {
  va_list args;
  va_start(args, format);
  printf("0.0\n");
  if (is_test) {
    fprintf(stderr, "translate:wrong\n");
  } else {
    fprintf(stderr, "Bug: Error in output: ");
    vfprintf(stderr, format, args);
    fprintf(stderr, "\n");
  }
  va_end(args);
  exit(is_test ? EXIT_CODE_INVALID_TEST_OUTPUT : EXIT_CODE_CHECKER_ERROR);
}

void read_input(FILE *file) {
  if (fscanf(file, "%d", &n) != 1) {
    input_fail("failed to read number of vertices");
  }
  if (!(1 <= n && n <= MAXN)) {
    input_fail("number of vertices is out of range");
  }
  if (fscanf(file, "%d", &m) != 1) {
    input_fail("failed to read number of edges");
  }
  if (!(0 <= m && m <= n * (n - 1) / 2)) {
    input_fail("number of edges is out of range");
  }
  for (int i = 0; i < m; i++) {
    int a, b;
    if (fscanf(file, "%d%d", &a, &b) != 2) {
      input_fail("failed to read edge #%d", i);
    }
    if (!(1 <= a && a < b && b <= n)) {
      input_fail("edge #%d is not valid", i);
    }
    a--;
    b--;
    if (neighbors[a][b]) {
      input_fail("duplicate edge #%d", i);
    }
    neighbors[a].set(b);
    neighbors[b].set(a);
  }
  fclose(file);
}

bool read_output(FILE *file, bool is_test) {
  int h1, h2, h3, h4;
  if (fscanf(file, "%d", &h1) != 1) {
    output_fail(is_test, "failed to read first number");
  }
  bool no_solution = (h1 == 0);
  if (!no_solution) {
    if (fscanf(file, "%d%d%d", &h2, &h3, &h4) != 3) {
      output_fail(is_test, "failed to read vertices");
    }
    if (
      !(
        h1 >= 1 && h1 <= n &&
        h2 >= 1 && h2 <= n &&
        h3 >= 1 && h3 <= n &&
        h4 >= 1 && h4 <= n
      )
    ) {
      output_fail(is_test, "vertices are out of range");
    }
    if (h1 == h2 || h1 == h3 || h1 == h4 || h2 == h3 || h2 == h4 || h3 == h4) {
      output_fail(is_test, "vertices in are not unique");
    }
    h1--;
    h2--;
    h3--;
    h4--;
    if (
      !(
        neighbors[h1][h2] &&
        neighbors[h2][h3] &&
        neighbors[h3][h4] &&
        neighbors[h4][h1]
      )
    ) {
      output_fail(is_test, "not a valid four-cycle");
    }
  }
  fclose(file);
  return no_solution;
}

int main(int argc, char **argv) {
  if (argc != 4) {
    fprintf(
      stderr,
      "Usage: ./check-output <input-file> <output-file> <test-output-file>\n"
    );
    return EXIT_CODE_CHECKER_ERROR;
  }

  FILE *input_file = fopen(argv[1], "r");
  if (!input_file) {
    perror("open input file");
    return EXIT_CODE_CHECKER_ERROR;
  }
  read_input(input_file);

  FILE *output_file = fopen(argv[2], "r");
  if (!output_file) {
    perror("open output file");
    return EXIT_CODE_CHECKER_ERROR;
  }
  bool output_no_solution = read_output(output_file, false);

  FILE *test_output_file = fopen(argv[3], "r");
  if (!test_output_file) {
    perror("open test output file");
    return EXIT_CODE_CHECKER_ERROR;
  }
  bool test_output_no_solution = read_output(test_output_file, true);

  if (output_no_solution == test_output_no_solution) {
    printf("1.0\n");
    fprintf(stderr, "translate:success\n");
    return EXIT_CODE_VALID_TEST_OUTPUT;
  } else {
    if (test_output_no_solution) {
      output_fail(true, "four-cycle not found although it exists");
      return EXIT_CODE_CHECKER_ERROR;
    } else {
      output_fail(false, "this four-cycle should not exist!");
      return EXIT_CODE_INVALID_TEST_OUTPUT;
    }
  }
}
